#include "NamedCounterFactory.h"

#include "EnumNamedCounterImpl.h"
#include "EnumNamedCounterDataImpl.h"
#include "NamedCounterIteratorImpl.h"

using std::string;

NamedCounterFactoryImpl::NamedCounterFactoryImpl(const size_t numCounters)
{
   size_t value = 0;

   for (int w = 'A'; w <= 'Z'; w++)
   {
      for (int x = 'A'; x <= 'Z'; x++)
      {
         for (int y = 'A'; y <= 'Z'; y++)
         {
            for (int z = 'A'; z <= 'Z'; z++)
            {
               if (value == numCounters)
               {
                  return;
               }

               char name[5] = { w, x, y, z, '\0' };
            
               AddNamedCounter(name, value++);
            }
         }
      }
   }
}

NamedCounterFactoryImpl::~NamedCounterFactoryImpl()
{
   CounterContainer::iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      (*it).second->Release();
      ++it;
   }
}

NamedCounter_ptr NamedCounterFactoryImpl::GetNamedCounter(
   const char* name)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   (*it).second->AddRef();

   return NamedCounter::_duplicate((*it).second->Data()->_this());
}

void NamedCounterFactoryImpl::AddNamedCounter(
   const char* name, 
   CORBA::Long initialValue)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it != m_counters.end())
   {
      // found

      throw NamedCounterFactory::CounterExists(name);
   }

   NamedCounterImpl *pCounter = new NamedCounterImpl(name, initialValue);

   m_counters.insert(CounterContainer::value_type(name, pCounter));
}

void NamedCounterFactoryImpl::RemoveNamedCounter(
   const char* name)
{
   CounterContainer::iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   // this erases the map entry between the name of the counter and
   // the pointer to the counter object, unfortunately we can't delete
   // the counter object itself as to do so would cause  

   (*it).second->Release();

   m_counters.erase(it);   
}
  

counterSeq *NamedCounterFactoryImpl::GetAll()
{
   const size_t numCounters = m_counters.size();

   counterSeq_var seq = new counterSeq(numCounters);

	seq->length(numCounters);

   size_t i = 0;

   CounterContainer::const_iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      NamedCounterImpl *pCounter = (*it).second->Data();

      pCounter->AddRef();

      seq[i] = NamedCounter::_duplicate(pCounter->_this());
    
      ++it;
      ++i;
   }

   return seq._retn();
}

EnumNamedCounter_ptr NamedCounterFactoryImpl::GetEnumerator()
{
   EnumNamedCounterImpl *pEnum = new EnumNamedCounterImpl(m_counters);

   return pEnum->_this();
}

counterDataSeq *NamedCounterFactoryImpl::GetAllReadOnly()
{
   const size_t numCounters = m_counters.size();

   counterDataSeq_var seq = new counterDataSeq(numCounters);

	seq->length(numCounters);

   size_t i = 0;

   CounterContainer::const_iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      NamedCounterImpl *pCounter = (*it).second->Data();

      struct CounterData data;

      data.name = pCounter->name();
      data.value = pCounter->value();

      seq[i] = data;

      ++it;
      ++i;
   }

   return seq._retn();
}

EnumNamedCounterData_ptr NamedCounterFactoryImpl::GetEnumeratorReadOnly()
{
   EnumNamedCounterDataImpl *pEnum = new EnumNamedCounterDataImpl(m_counters);

   return pEnum->_this();
}

void NamedCounterFactoryImpl::GetIterator(
   CORBA::ULong how_many, 
   counterSeq_out theCounters, 
   _CORBA_ObjRef_OUT_arg< _objref_NamedCounterIterator, NamedCounterIterator_Helper > it)
{
   NamedCounterIteratorImpl *pIt = new NamedCounterIteratorImpl(m_counters);

   pIt->next_n(how_many, theCounters);

   if (pIt->AtEnd())
   {
      pIt->destroy();

      it = NamedCounterIterator::_nil();
   }
   else
   {
        it = pIt->_this();
        //pIt->_remove_ref();
   }

   // create an iterator
   // if how_many is non zero, call next_n on it to get back the seq full of
   // counters for us to return
   // check if the iterator is at the end, if so, destroy it and return a 
   // nill object reference for it.
}

